from enum import Enum
from typing import Annotated
import typer

from knoc.workflows import workflows

app = typer.Typer(no_args_is_help=True)

enum_workflows = Enum("workflows", {k: k for k in workflows})


@app.command()
def list():
    """List all workflows"""

    typer.echo("\n".join([name for name in workflows]))


@app.command()
def run(workflow: enum_workflows):  # type: ignore
    """Run a workflow"""
    if workflow.name not in enum_workflows:
        typer.echo(
            f"Unknown workflow {workflow.name}. Use 'list' to view all workflows"
        )
        raise typer.Exit(1)
    wf_setting = workflows[workflow.name]
    result = wf_setting.wf.run()  # type: ignore
    typer.echo(result)
