from datetime import datetime
from pydantic import BaseModel
from hatchet_sdk import Context

from knoc.hatchet_client import hatchet
from knoc import __version__


class SimpleInput(BaseModel):
    friendly: bool = True
    name: str = ""
    # Note: This should ideally be:
    # name: str | None = None
    # ... But optional fields are not well supported by pydantic v2.
    # See:
    # * https://github.com/pydantic/pydantic/issues/7161
    # * https://github.com/pydantic/pydantic/issues/8394
    # Proper resolution will apparently take time as it depends on
    # https://peps.python.org/pep-0661/
    # ... and is planned for v2.12.


class Greeter(BaseModel):
    message: str


class Time(BaseModel):
    time: datetime


class Version(BaseModel):
    version: str


greeting_wf = hatchet.workflow(
    name="Greeting",
    description="Does nothing much, but says hi and gives time",
    input_validator=SimpleInput,
    version=__version__,
)


@hatchet.task(input_validator=SimpleInput, description="Greet as a task")
async def say_hi(input: SimpleInput, ctx: Context) -> Greeter:
    ctx.log(f"Greting {input.name}")
    greeting = "Ahoy" if input.friendly else "Hello"
    who = input.name or "John Doe"
    return Greeter(message=f"{greeting}, {who}!")


@hatchet.task(input_validator=SimpleInput, description="Give current time")
async def give_time(input: SimpleInput, ctx: Context) -> Time:
    assert input
    ctx.log(f"Checking time")
    return Time(time=datetime.now())


@hatchet.task(input_validator=SimpleInput, description="Give the version of the worker")
async def give_version(input: SimpleInput, ctx: Context) -> Version:
    assert input
    ctx.log(f"Version: {__version__}")
    return Version(version=__version__)


greeting_wf.add_task(say_hi)
greeting_wf.add_task(give_time)
greeting_wf.add_task(give_version)
